/*
 * Project name:
     FreeRTOS ADC Display
 * Copyright:
     (c) Mikroelektronika, 2018.
 * Revision History:
     20180320:
       - Initial release;
 * Description:
     This code uses buttons and an analog input for inputting user data,
     and shows the data on TFT display and LEDs.

     TFT will constantly be displaying the ADC input,
     as well as indicate which button was pressed.
     PORTA and PORTC LEDs will be displaying the value of the ADC input as well,
     and will be blinking with a frequency
     that can be adjusted by pressing the appropriate buttons.

     The on-board potentiometer can be used to change the value
     of the ADC input. Buttons RF4, RF5, RF12, and RF13 can be used
     to change the blinking frequency of the LEDs that display the ADC input.
 * Test configuration:
     MCU:             P33EP512MU810
     Dev.Board:       EasyPIC Fusion v7
                      http://www.mikroe.com/easypic-fusion/
     Oscillator:      XT-PLL, 80.0000MHz
     Ext. Modules:    EasyTFT display
     SW:              mikroC PRO for dsPIC
                      http://www.mikroe.com/mikroc/dspic/
 * NOTES:
     - Connect jumper J8 to RB0 (ADC channel 0 input).
     - Enable button press level SW10.6 for PORTF.
     - Pull down on PORTF pins 4, 5, 12, and 13.
     - Turn on LEDs on PORTA&C at SW15.
     - Turn ON TFT BCK LIGHT switch at SW11.1.
 */

// Includes

#include "main.h"

// Public Function Definitions

// Main function, which creates the tasks and starts the scheduler.
void main()
{
    // Setting output frequency to 80MHz
    PLLPRE_0_bit = 0;
    PLLPRE_1_bit = 0;
    PLLPRE_2_bit = 0;
    PLLPRE_3_bit = 0;
    PLLPRE_4_bit = 0;

    PLLFBD = 38;

    PLLPOST_0_bit = 0;
    PLLPOST_1_bit = 0;

    // Create init task.
    xTaskCreate(
        (TaskFunction_t)initTask,
        "Init Task",
        configMINIMAL_STACK_SIZE,
        NULL,
        1,
        NULL
    );

    // Start the RTOS scheduler.
    vTaskStartScheduler();

    // Will never reach here.
    while (1);
}