/*
    FreeRTOS V9.0.1 - Copyright (C) 2017 Real Time Engineers Ltd.
    All rights reserved

    VISIT http://www.FreeRTOS.org TO ENSURE YOU ARE USING THE LATEST VERSION.

    This file is part of the FreeRTOS distribution.

    FreeRTOS is free software; you can redistribute it and/or modify it under
    the terms of the GNU General Public License (version 2) as published by the
    Free Software Foundation >>>> AND MODIFIED BY <<<< the FreeRTOS exception.

    ***************************************************************************
    >>!   NOTE: The modification to the GPL is included to allow you to     !<<
    >>!   distribute a combined work that includes FreeRTOS without being   !<<
    >>!   obliged to provide the source code for proprietary components     !<<
    >>!   outside of the FreeRTOS kernel.                                   !<<
    ***************************************************************************

    FreeRTOS is distributed in the hope that it will be useful, but WITHOUT ANY
    WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
    FOR A PARTICULAR PURPOSE.  Full license text is available on the following
    link: http://www.freertos.org/a00114.html

    ***************************************************************************
     *                                                                       *
     *    FreeRTOS provides completely free yet professionally developed,    *
     *    robust, strictly quality controlled, supported, and cross          *
     *    platform software that is more than just the market leader, it     *
     *    is the industry's de facto standard.                               *
     *                                                                       *
     *    Help yourself get started quickly while simultaneously helping     *
     *    to support the FreeRTOS project by purchasing a FreeRTOS           *
     *    tutorial book, reference manual, or both:                          *
     *    http://www.FreeRTOS.org/Documentation                              *
     *                                                                       *
    ***************************************************************************

    http://www.FreeRTOS.org/FAQHelp.html - Having a problem?  Start by reading
    the FAQ page "My application does not run, what could be wrong?".  Have you
    defined configASSERT()?

    http://www.FreeRTOS.org/support - In return for receiving this top quality
    embedded software for free we request you assist our global community by
    participating in the support forum.

    http://www.FreeRTOS.org/training - Investing in training allows your team to
    be as productive as possible as early as possible.  Now you can receive
    FreeRTOS training directly from Richard Barry, CEO of Real Time Engineers
    Ltd, and the world's leading authority on the world's leading RTOS.

    http://www.FreeRTOS.org/plus - A selection of FreeRTOS ecosystem products,
    including FreeRTOS+Trace - an indispensable productivity tool, a DOS
    compatible FAT file system, and our tiny thread aware UDP/IP stack.

    http://www.FreeRTOS.org/labs - Where new FreeRTOS products go to incubate.
    Come and try FreeRTOS+TCP, our new open source TCP/IP stack for FreeRTOS.

    http://www.OpenRTOS.com - Real Time Engineers ltd. license FreeRTOS to High
    Integrity Systems ltd. to sell under the OpenRTOS brand.  Low cost OpenRTOS
    licenses offer ticketed support, indemnification and commercial middleware.

    http://www.SafeRTOS.com - High Integrity Systems also provide a safety
    engineered and independently SIL3 certified version for use in safety and
    mission critical applications that require provable dependability.

    1 tab == 4 spaces!
*/

/*
        Changes from V4.2.1

        + Introduced the configKERNEL_INTERRUPT_PRIORITY definition.
*/

/*-----------------------------------------------------------
 * Implementation of functions defined in portable.h for the PIC24 port.
 *----------------------------------------------------------*/

/* Scheduler include files. */
#include "FreeRTOS.h"
#include "task.h"

/* From port.c. */
extern void *pxCurrentTCB;

/* Task function pointer for pragma indfuncall. */
void (*taskFunction)(void*);

/* Hardware specifics. */
#define portBIT_SET 1
#define portTIMER_PRESCALE 16
#define portINITIAL_SR        0

/* Defined for backward compatability with project created prior to
FreeRTOS.org V4.3.0. */
#ifndef configKERNEL_INTERRUPT_PRIORITY
        #define configKERNEL_INTERRUPT_PRIORITY 1
#endif

/* Use _T1Interrupt as the interrupt handler name if the application writer has
not provided their own. */
#ifndef configTICK_INTERRUPT_HANDLER
        #define configTICK_INTERRUPT_HANDLER _T1Interrupt
#endif /* configTICK_INTERRUPT_HANDLER */

/* The program counter is only 23 bits. */
#define portUNUSED_PR_BITS        0x7f

/* Records the nesting depth of calls to portENTER_CRITICAL(). */
UBaseType_t uxCriticalNesting = 0xef;

#if configKERNEL_INTERRUPT_PRIORITY != 1
        #error If configKERNEL_INTERRUPT_PRIORITY is not 1 then the #32 in the following macros needs changing to equal the portINTERRUPT_BITS value, which is ( configKERNEL_INTERRUPT_PRIORITY << 5 )
#endif

#if defined( __P24__ )

    #ifdef __HAS_EDS__
        #define portRESTORE_CONTEXT()                                          \
            /* Restore the stack pointer for the task. */                      \
            asm MOV   _pxCurrentTCB, W0;                         \
            asm MOV   [W0], W15;                                               \
            /* Restore the critical nesting counter for the task. */           \
            asm POP   W0;                                                      \
            asm MOV   W0, _uxCriticalNesting;                                  \
            asm POP   DSWPAG;                                                  \
            asm POP   DSRPAG;                                                  \
            asm POP   CORCON;                                                  \
            asm POP   TBLPAG;                                                  \
            /* Restore the registers from the stack. */                        \
            asm POP   RCOUNT;                                                  \
            asm POP   W14;                                                     \
            asm POP.D W12;                                                     \
            asm POP.D W10;                                                     \
            asm POP.D W8;                                                      \
            asm POP.D W6;                                                      \
            asm POP.D W4;                                                      \
            asm POP.D W2;                                                      \
            asm POP.D W0;                                                      \
            asm POP   SR
    #else /* __HAS_EDS__ */
        #define portRESTORE_CONTEXT()                                          \
            /* Restore the stack pointer for the task. */                      \
            asm MOV   _pxCurrentTCB, W0;                         \
            asm MOV   [W0], W15;                                               \
            /* Restore the critical nesting counter for the task. */           \
            asm POP   W0;                                                      \
            asm MOV   W0, _uxCriticalNesting;                                  \
            asm POP   PSVPAG;                                                  \
            asm POP   CORCON;                                                  \
            asm POP   TBLPAG;                                                  \
            /* Restore the registers from the stack. */                        \
            asm POP   RCOUNT;                                                  \
            asm POP   W14;                                                     \
            asm POP.D W12;                                                     \
            asm POP.D W10;                                                     \
            asm POP.D W8;                                                      \
            asm POP.D W6;                                                      \
            asm POP.D W4;                                                      \
            asm POP.D W2;                                                      \
            asm POP.D W0;                                                      \
            asm POP   SR
    #endif /* __HAS_EDS__ */
#endif /* MPLAB_PIC24_PORT */

#if defined( __P30__ ) || defined( __P33__ )

    #ifdef __HAS_EDS__
        #define portRESTORE_CONTEXT()                                          \
            /* Restore the stack pointer for the task. */                      \
            asm MOV   _pxCurrentTCB, W0;                         \
            asm MOV   [W0], W15;                                               \
            /* Restore the critical nesting counter for the task. */           \
            asm POP   W0;                                                      \
            asm MOV   W0, _uxCriticalNesting;                                  \
            asm POP   DSWPAG;                                                  \
            asm POP   DSRPAG;                                                  \
            asm POP   CORCON;                                                  \
            /* asm POP   DOENDH;                                                  \
            asm POP   DOENDL;                                                  \
            asm POP   DOSTARTH;                                                \
            asm POP   DOSTARTL;                                                \
            asm POP   DCOUNT; */                                                  \
            asm POP   ACCBU;                                                   \
            asm POP   ACCBH;                                                   \
            asm POP   ACCBL;                                                   \
            asm POP   ACCAU;                                                   \
            asm POP   ACCAH;                                                   \
            asm POP   ACCAL;                                                   \
            asm POP   TBLPAG;                                                  \
            /* Restore the registers from the stack. */                        \
            asm POP   RCOUNT;                                                  \
            asm POP   W14;                                                     \
            asm POP.D W12;                                                     \
            asm POP.D W10;                                                     \
            asm POP.D W8;                                                      \
            asm POP.D W6;                                                      \
            asm POP.D W4;                                                      \
            asm POP.D W2;                                                      \
            asm POP.D W0;                                                      \
            asm POP   SR
    #else /* __HAS_EDS__ */
        #define portRESTORE_CONTEXT()                                          \
            /* Restore the stack pointer for the task. */                      \
            asm MOV   _pxCurrentTCB, W0;                         \
            asm MOV   [W0], W15;                                               \
            /* Restore the critical nesting counter for the task. */           \
            asm POP   W0;                                                      \
            asm MOV   W0, _uxCriticalNesting;                                  \
            asm POP   PSVPAG;                                                  \
            asm POP   CORCON;                                                  \
            /* asm POP   DOENDH;                                                  \
            asm POP   DOENDL;                                                  \
            asm POP   DOSTARTH;                                                \
            asm POP   DOSTARTL;                                                \
            asm POP   DCOUNT; */                                                  \
            asm POP   ACCBU;                                                   \
            asm POP   ACCBH;                                                   \
            asm POP   ACCBL;                                                   \
            asm POP   ACCAU;                                                   \
            asm POP   ACCAH;                                                   \
            asm POP   ACCAL;                                                   \
            asm POP   TBLPAG;                                                  \
            /* Restore the registers from the stack. */                        \
            asm POP   RCOUNT;                                                  \
            asm POP   W14;                                                     \
            asm POP.D W12;                                                     \
            asm POP.D W10;                                                     \
            asm POP.D W8;                                                      \
            asm POP.D W6;                                                      \
            asm POP.D W4;                                                      \
            asm POP.D W2;                                                      \
            asm POP.D W0;                                                      \
            asm POP   SR
    #endif /* __HAS_EDS__ */

#endif /* MPLAB_DSPIC_PORT */

#define hash #

#ifndef portRESTORE_CONTEXT
    #error Unrecognised device selected

    /* Note:  dsPIC parts with EDS are not supported as there is no easy way to
    recover the hardware stacked copies for DOCOUNT, DOHIGH, DOLOW. */
#endif

/*
 * Yield the processor.
 */
void vPortYield( void )
{
    #ifdef MIKRO_ICD
    asm ULNK;
    #endif

    asm {
        /* Save the SR used by the task.... */
        PUSH   SR
        /* ....then disable interrupts. */
        PUSH   W0
        MOV    #32, W0
        MOV    W0, SR
        /* Save registers to the stack. */
        PUSH   W1
        PUSH.D W2
        PUSH.D W4
        PUSH.D W6
        PUSH.D W8
        PUSH.D W10
        PUSH.D W12
        PUSH   W14
        PUSH   RCOUNT
        PUSH   TBLPAG
    }
    
    #if defined(__P30__) || defined(__P33__)
    asm {
        PUSH   ACCAL
        PUSH   ACCAH
        PUSH   ACCAU
        PUSH   ACCBL
        PUSH   ACCBH
        PUSH   ACCBU
    }
    
    #ifndef __HAS_EDS__
    asm {
        PUSH   DCOUNT
        PUSH   DOSTARTL
        PUSH   DOSTARTH
        PUSH   DOENDL
        PUSH   DOENDH
    }
    #endif /* __HAS_EDS__ */
    #endif /* defined(__P30__) || defined(__P33__) */

    asm PUSH   CORCON;
    #ifdef __HAS_EDS__
    asm PUSH   DSRPAG;
    asm PUSH   DSWPAG;
    #else
    asm PUSH   PSVPAG;
    #endif /* __HAS_EDS__ */

    asm {
        /* Save the critical nesting counter for the task. */
        MOV    _uxCriticalNesting, W0
        PUSH   W0
        /* Save the new top of stack into the TCB. */
        MOV    _pxCurrentTCB, W0
        MOV    W15, [W0]

        call   _vTaskSwitchContext

        /* Restore the stack pointer for the task. */
        MOV    _pxCurrentTCB, W0
        MOV    [W0], W15
        /* Restore the critical nesting counter for the task. */
        POP    W0
        MOV    W0, _uxCriticalNesting
    }
    
    #ifdef __HAS_EDS__
    asm POP DSWPAG;
    asm POP DSRPAG;
    #else
    asm POP PSVPAG;
    #endif /* __HAS_EDS__ */
    asm POP    CORCON;

    #if defined(__P30__) || defined(__P33__)
    #ifndef __HAS_EDS__
    asm {
        POP    DOENDH
        POP    DOENDL
        POP    DOSTARTH
        POP    DOSTARTL
        POP    DCOUNT
    }
    #endif
    
    asm {
        POP    ACCBU
        POP    ACCBH
        POP    ACCBL
        POP    ACCAU
        POP    ACCAH
        POP    ACCAL
    }
    #endif /* defined(__P30__) || defined(__P33__) */
    
    asm {
        POP    TBLPAG
        /* Restore the registers from the stack. */
        POP    RCOUNT
        POP    W14
        POP.D  W12
        POP.D  W10
        POP.D  W8
        POP.D  W6
        POP.D  W4
        POP.D  W2
        POP.D  W0
        POP    SR
    }

    #ifdef MIKRO_ICD
    asm LNK #0;
    #endif
}

/*
 * Setup the timer used to generate the tick interrupt.
 */
void vApplicationSetupTickTimerInterrupt( void );

/*
 * See header file for description.
 */
StackType_t *pxPortInitialiseStack( StackType_t *pxTopOfStack, TaskFunction_t pxCode, void *pvParameters )
{
    uint16_t usCode;
    UBaseType_t i;

    const StackType_t xInitialStack[] =
    {
        0x1111, /* W1 */
        0x2222, /* W2 */
        0x3333, /* W3 */
        0x4444, /* W4 */
        0x5555, /* W5 */
        0x6666, /* W6 */
        0x7777, /* W7 */
        0x8888, /* W8 */
        0x9999, /* W9 */
        0xaaaa, /* W10 */
        0xbbbb, /* W11 */
        0xcccc, /* W12 */
        0xdddd, /* W13 */
        0xeeee, /* W14 */
        0xcdce, /* RCOUNT */
        0xabac, /* TBLPAG */

        /* dsPIC specific registers. */
        #if defined(__P30__) || defined(__P33__)
        0x0202, /* ACCAL */
        0x0303, /* ACCAH */
        0x0404, /* ACCAU */
        0x0505, /* ACCBL */
        0x0606, /* ACCBH */
        0x0707, /* ACCBU */
        
        #ifndef __HAS_EDS__
        0x0808, /* DCOUNT */
        0x090a, /* DOSTARTL */
        0x1010, /* DOSTARTH */
        0x1110, /* DOENDL */
        0x1212, /* DOENDH */
        #endif
        #endif
    };

    /* Setup the stack as if a yield had occurred.

    Save the low bytes of the program counter. */
    usCode = ( uint16_t ) pxCode;
    *pxTopOfStack = ( StackType_t ) usCode;
    pxTopOfStack++;

    /* Save the high byte of the program counter.  This will always be zero
    here as it is passed in a 16bit pointer.  If the address is greater than
    16 bits then the pointer will point to a jump table. */
    *pxTopOfStack = ( StackType_t ) 0;
    pxTopOfStack++;

    /* Status register with interrupts enabled. */
    *pxTopOfStack = portINITIAL_SR;
    pxTopOfStack++;

    /* Parameters are passed in W0. */
    *pxTopOfStack = ( StackType_t ) pvParameters;
    pxTopOfStack++;

    for( i = 0; i < ( sizeof( xInitialStack ) / sizeof( StackType_t ) ); i++ )
    {
        *pxTopOfStack = xInitialStack[ i ];
        pxTopOfStack++;
    }

    *pxTopOfStack = CORCON;
    pxTopOfStack++;

    #if defined(__HAS_EDS__)
    *pxTopOfStack = DSRPAG;
    pxTopOfStack++;
    *pxTopOfStack = DSWPAG;
    pxTopOfStack++;
    #else /* __HAS_EDS__ */
    *pxTopOfStack = PSVPAG;
    pxTopOfStack++;
    #endif /* __HAS_EDS__ */

    /* Finally the critical nesting depth. */
    *pxTopOfStack = 0x00;
    pxTopOfStack++;

    return pxTopOfStack;
}
/*-----------------------------------------------------------*/

BaseType_t xPortStartScheduler( void )
{
    /* Setup stack pointer limit. */
    SPLIM = 0x7FFE;

    /* Setup a timer for the tick ISR. */
    vApplicationSetupTickTimerInterrupt();

    /* Restore the context of the first task to run. */
    portRESTORE_CONTEXT();

    /* Simulate the end of the yield function. */
    asm return;

    /* Should not reach here. */
    return pdTRUE;
}

/* Pragma to enable linking of functions that are not explicitly called,
i.e. tasks. */
#pragma indfuncall xPortStartScheduler taskFunction
/*-----------------------------------------------------------*/

void vPortEndScheduler( void )
{
    /* Not implemented in ports where there is nothing to return to.
    Artificially force an assert. */
    configASSERT( uxCriticalNesting == 1000UL );
}
/*-----------------------------------------------------------*/

/*
 * Setup a timer for a regular tick.
 */
void vApplicationSetupTickTimerInterrupt( void )
{
    const uint32_t ulCompareMatch =
        ( ( configCPU_CLOCK_HZ / portTIMER_PRESCALE ) /
        configTICK_RATE_HZ ) - 1;

    /* Prescale of 8. */
    T1CON = 0;
    TMR1 = 0;

    PR1 = ( uint16_t ) ulCompareMatch;

    /* Setup timer 1 interrupt priority. */
    IPC0bits.T1IP = configKERNEL_INTERRUPT_PRIORITY;

    /* Clear the interrupt as a starting condition. */
    IFS0bits.T1IF = 0;

    /* Enable the interrupt. */
    IEC0bits.T1IE = 1;

    /* Setup the prescale value. */
    T1CONbits.TCKPS0 = 1;
    T1CONbits.TCKPS1 = 0;

    /* Start the timer. */
    T1CONbits.TON = 1;
}
/*-----------------------------------------------------------*/

void vPortEnterCritical( void )
{
    portDISABLE_INTERRUPTS();
    uxCriticalNesting++;
}
/*-----------------------------------------------------------*/

void vPortExitCritical( void )
{
    configASSERT( uxCriticalNesting );
    uxCriticalNesting--;
    if( uxCriticalNesting == 0 )
    {
        portENABLE_INTERRUPTS();
    }
}
/*-----------------------------------------------------------*/

void configTICK_INTERRUPT_HANDLER( void ) iv IVT_ADDR_T1INTERRUPT ics ICS_AUTO
{
    /* Clear the timer interrupt. */
    IFS0bits.T1IF = 0;

    if( xTaskIncrementTick() != pdFALSE )
    {
        portYIELD();
    }
}